# Hangman
from random import randint
from Tkinter import *

#read list of words from a file
f = open("words.txt")
words = f.readlines()
# remove white-space and newline characters
words = [w.strip() for w in words]
for w in words:
    # remove blanks from the list
    if w == "": words.remove(w)

# game over
def end_game():
    global guesses, word
    if lives == 0:
        instructions.set("It was '"+word+"' - press any key.")
    else:
        instructions.set("Well done - it is '"+word+"'!")

# handler for keypresses from the form
def keypress(event):
    global guesses, display, word, the_end, lives
    # Boolean variable - is the letter pressed in the word?
    letter_found = False
    # string comparisons are case-sensitive, so work in lower case
    letter = event.char.lower()
    if the_end:
        start_again()
    else:
        # turn non-printing characters into a space
        if len(letter) == 0: letter = " "
        # if it's a letter and hasn't been tried before...
        if ord(letter) > 96 and ord(letter) < 123 and guesses.find(letter, 0) < 0:
            instructions.set(letter)
            # add letter to previous guesses
            guesses += letter
            # check whether the letter is in the word
            position = word.find(letter, 0)
            # the letter might occur more than one, so repeat
            while position >= 0:
                letter_found = True
                # if letter is found in the word, insert at the corresponding position in blanks
                display = display[0:position] + word[position:position+1] + display[position+1:]
                # look for the letter again starting after the position where it was found last time
                position = word.find(letter, position + 1)
            # update the word displayed on the form
            blanks.set(display)
            if letter_found:
                instructions.set("The letter "+letter.upper()+" is in the word.")
            else:
                lives -= 1
                score.set("Score: "+str(lives))
                instructions.set("Sorry, there is no "+letter.upper()+" - lose a point!")
            # check for remaining lives and any blanks left in the word
            if lives == 0 or display.find("-",0) < 0:
                the_end = True
                end_game()
        else:
            # the letter has been tried before
            if guesses.find(letter, 0) >= 0:
                instructions.set("You've tried "+letter.upper()+" before - please try again.")
            # the key press wasn't a letter
            else:
                instructions.set("That's not a letter - please try again.")

def start_again():
    global guesses, display, word, the_end, lives
    # pick a random word from the list and convert to lower case
    word = (words[randint(0,len(words)-1)]).lower()
    # letter tried in this game
    guesses = ""
    # the word as displayed on the form
    display = ""
    # add a hyphen to the display for each letter in the word
    display = "-" * len(word)
    # display the word
    blanks.set(display)
    # display the instructions
    instructions.set("Press a letter on the keyboard to begin.")
    lives = 6
    the_end = False
    # display the number of lives left
    score.set("Score: "+str(lives))
    # display the number of letters in the word
    letters.set("Letters: "+str(len(word)))

# create the hangman window
hangman = Tk()
hangman.resizable(width=FALSE, height=FALSE)
hangman.title("Hangman")
# send keypresses from the form to the "keypress" function
hangman.bind("<Key>",keypress)

# create display labels and buttons
instructions = StringVar()
lbl_instructions = Label(hangman, textvariable=instructions, fg="black", width=32, font=("Helvetica", 9), anchor=CENTER).grid(row=2, column=0, columnspan=3)
blanks = StringVar()
lbl_display = Label(hangman, textvariable=blanks, fg="black", width=16, font=("Courier", 18), anchor=CENTER).grid(row=1, column=0, columnspan=3)
score = StringVar()
lbl_score = Label(hangman, textvariable=score, fg="black", width=12, font=("Helvetica", 9, "italic"), anchor=W).grid(row=0, column=0)
letters = StringVar()
lbl_letters = Label(hangman, textvariable=letters, fg="black", width=12, font=("Helvetica", 9, "italic"), anchor=E).grid(row=0, column=2)
buttonReset = Button(hangman, text="Reset", fg="red", command=lambda: start_again(), width=16, font=("Helvetica", 11), anchor=CENTER).grid(row=3, column=0, columnspan=3)

# initialise scores, pick word, etc.
start_again()

# start the event loop
hangman.mainloop()
